package jp.co.mki.guide.bean;

import java.util.Map;
import jp.co.mki.guide.common.*;

/**
 * Data Bean for Blast View
 * @author Y.KIDO
 * @version 0.0.1
 */
public class BlastBean extends BaseJobBean {

    /** BLAST's Program Name */
    private String programName = null;

    /**
     * BLAST's Program Name setter
     * @param value Program Name
     */
    public void setProgramName(String value){ this.programName = value; }

    /**
     * BLAST's Program Name getter
     * @return String Program Name
     */
    public String getProgramName(){ return this.programName; }

    /**
     * Program Name Array (JavaScript format) getter
     * <pre>
     * Example return value:
     * programArray[0] = new Array();
     * programArray[0][0] = "blastn";
     * programArray[0][1] = "blastp";
     * programArray[0][2] = "blastx";
     * programArray[0][3] = "tblastn";
     * see: blast.jsp
     * </pre>
     * @return String Program Name Array
     */
    public String getProgramArrayForJS() {
        StringBuffer buf = new StringBuffer();
        String[] hosts = CommonHosts.getHostsArray();
        for ( int i = 0; i < hosts.length; i++ ) {
            Map prop = CommonProperties.getHostProperties(hosts[i]);
            buf.append("programArray[" + i + "] = new Array();\n");
            for ( int j = 0; true; j++ ) {
                String value = (String) prop.get("BLAST_PROGRAMS." + j);
                if ( value == null || value.equals("EOA") ) break;
                buf.append("programArray[" + i + "][" + j + "] = \"" + value + "\";\n");
            }
        }
        return new String(buf);
    }

    /** Bioinfomatics DataBase Name */
    private String dbName = "";

    /**
     * Bioinfomatics DB Name setter
     * @param value DB Name
     */
    public void setDBName(String value){ this.dbName = value; }

    /**
     * Bioinfomatics DB Name getter
     * @return String DB name
     */
    public String getDBName(){ return this.dbName; }

    /**
     * DataBase Name Array (JavaScript format) getter
     * <pre>
     * Example return value:
     * dbArray[1][0] = "/data/blast/est.01";
     * dbArray[1][1] = "/data/blast/gss";
     * dbArray[1][2] = "/data/blast/htgs.00";
     * dbArray[1][3] = "/data/blast/htgs.01";
     * dbArray[1][4] = "/data/blast/human_genomic";
     * dbArray[1][5] = "/data/blast/nr";
     * dbArray[1][6] = "/data/blast/nt";
     * see: blast.jsp
     * </pre>
     * @return String DataBase Name Array
     */
    public String getDBArrayForJS() {
        StringBuffer buf = new StringBuffer();
        String[] hosts = CommonHosts.getHostsArray();
        for ( int i = 0; i < hosts.length; i++ ) {
            Map prop = CommonProperties.getHostProperties(hosts[i]);
            buf.append("dbArray[" + i + "] = new Array();\n");
            int j = 0;
            for ( j = 0; true; j++ ) {
                String value = (String) prop.get("BLAST_DB." + j);
                if ( value == null || value.equals("EOA") ) break;
                buf.append("dbArray[" + i + "][" + j + "] = \"" + value + "\";\n");
            }

            // 2003.7.10 Y.Kido add.
            System.err.println("BlastBean: user name:[" + super.getUserName() + "]");
            for ( ; true; j++) {
                System.err.println(super.getUserName() + ".BLAST_DB." + j + " is ready!");
                String value = (String) prop.get(super.getUserName() + ".BLAST_DB." + j);
                if ( value == null || value.equals("EOA") ) break;
                buf.append("dbArray[" + i + "][" + j + "] = \"" + value + "\";\n");
            }
        }
        return new String(buf);
    }

    /** Filter is BLAST' param */
    private String filter = "true";

    /**
     * Filter String setter
     * @param value filter string
     */
    public void setFilter(String value) { this.filter = value; }

    /**
     * Filter String getter
     * @return String Filter string
     */
    public String getFilter() { return this.filter; }

    /** Expect is BLAST' param */
    private String expect = "";

    /**
     * Expect value setter
     * @param value expect
     */
    public void setExpect(String value) { this.expect = value; }

    /**
     * Expect value getter
     * @return String expect
     */
    public String getExpect() { return this.expect; }

    /** Matrix File Name (protein only) */
    private String matrixFileName = null;

    /**
     * Matrix File Name setter
     * @param value Matrix File name
     */
    public void setMatrixFileName(String value) { this.matrixFileName = value; }

    /**
     * Matrix File Name getter
     * @return String Matrix File Name
     */
    public String getMatrixFileName(){ return this.matrixFileName; }

    /**
     * Matrix Name Array (JavaScript format) getter
     * <pre>
     * Example return value:
     * matrixArray[0][0] = "default";
     * matrixArray[0][1] = "/bg/home/blast/data/BLOSUM45";
     * matrixArray[0][2] = "/bg/home/blast/data/BLOSUM62";
     * matrixArray[0][3] = "/bg/home/blast/data/BLOSUM80";
     * matrixArray[0][4] = "/bg/home/blast/data/PAM30";
     * matrixArray[0][5] = "/bg/home/blast/data/PAM70";
     * see: blast.jsp
     * </pre>
     * @return String Matrix Name Array
     */
    public String getMatrixArrayForJS() {
        StringBuffer buf = new StringBuffer();
        String[] hosts = CommonHosts.getHostsArray();
        for ( int i = 0; i < hosts.length; i++ ) {
            Map prop = CommonProperties.getHostProperties(hosts[i]);
            buf.append("matrixArray[" + i + "] = new Array();\n");
            for ( int j = 0; true; j++ ) {
                String value = (String) prop.get("BLAST_MATRIX_FILE." + j);
                if ( value == null || value.equals("EOA") ) break;
                buf.append("matrixArray[" + i + "][" + j + "] = \"" + value + "\";\n");
            }
        }
        return new String(buf);
    }

    /** Open Gap is BLAST's advanced param */
    private String openGap = "";

    /**
     * OpenGap setter
     * @param value Open Gap
     */
    public void setOpenGap(String value) { this.openGap = value; }

    /**
     * OpenGap getter
     * @return String Open Gap
     */
    public String getOpenGap() { return this.openGap; }

    /** ExtendGap is BLAST's advanced param */
    private String extendGap = "";

    /**
     * ExtendGap setter
     * @param value ExtendGap
     */
    public void setExtendGap(String value) { this.extendGap = value; }

    /**
     * ExtendGap getter
     * @return String ExtendGap
     */
    public String getExtendGap() { return this.extendGap; }

    /** Strand Value is BLAST's advanced param (Nucleic Only) */
    private String strand = "";
    /**
     * Strand Value setter
     * @param value Strand value
     */
     public void setStrand(String value){ this.strand = value; }

    /**
     * Strand Value getter
     * @return String Strand value
     */
     public String getStrand() { return this.strand; }


    /** BLAST's param more option */
    private String moreOption = "";

    /**
     * More Option setter
     * @param value More Option
     */
     public void setMoreOption(String value) { this.moreOption = value; }

    /**
     * More Option getter
     * @return String More Option
     */
     public String getMoreOption() { return this.moreOption; }
}