package jp.co.mki.guide.common;

import java.io.*;
import java.util.*;

/**
 * Job List Database
 * @author Y.KIDO
 * @version 0.0.1
 */
public final class JobDatabase {

    private JobDatabase() {};

    private static List list = loadList();

    private static final synchronized List loadList() {
        List database = new Vector();
        String fileName = CommonProperties.getProperty("LOCAL_WORK_DIR") + "/joblist";

        try {
            // Load Database File
            ObjectInputStream ois = new ObjectInputStream(new FileInputStream(fileName));
            database.addAll( (List) ois.readObject() );
            ois.close();
        } catch ( FileNotFoundException fex ) {
            System.err.println( fileName + " not found." );
        } catch ( Exception ex ) {
            ex.printStackTrace();
        }
        return database;
    }

    public static List getList(String usrName) {
        if ( list == null )
            list = loadList();

        List usrList = new Vector();
        for (Iterator it = list.iterator(); it.hasNext(); ) {
            BaseJobBean obj = (BaseJobBean) it.next();
            if ( obj.getUserName() != null && obj.getUserName().equals( usrName ) )
                 usrList.add(obj);
        }
        return usrList;
    }

    public static void add(BaseJobBean bean) {
        if ( list == null )
            list = loadList();
        list.add(bean);
        writeList();
    }

    public static void remove(BaseJobBean bean) {
        if ( list == null )
            list = loadList();
        list.remove(bean);
        writeList();
    }

    protected static void write() {
        writeList();
    }

    private static final synchronized void writeList() {
        if ( list == null )
            list = new Vector();

        File file = new File(CommonProperties.getProperty("LOCAL_WORK_DIR") + "/joblist");
        try {
            if ( !file.exists() )
                file.createNewFile();

            java.io.ObjectOutputStream oos = new ObjectOutputStream(new FileOutputStream(file));
            oos.writeObject(list);
            oos.flush();
            oos.close();
        } catch ( Exception ex ) {
            ex.printStackTrace();
        }
    }
}
