package jp.co.mki.guide.model;

import java.util.*;
import java.io.*;
import javax.servlet.http.HttpSession;
import org.globus.security.*;
import org.globus.gram.GramJobListener;
import org.globus.gram.GramJob;
import org.globus.io.ftp.FTPClient;
import org.globus.io.ftp.GridFTPClient;
import org.globus.io.ftp.TransferProgressListener;
import jp.co.mki.guide.common.*;
import jp.co.mki.guide.bean.BlastBean;

/**
 * @author Y.KIDO
 * @version 0.0.1 KIDO 2003/04/20 created this class
 * @version 0.0.2 KIDO 2003/05/22 changed construct parameter
 */
public class BlastJobListener implements GramJobListener, TransferProgressListener {

    private BlastBean bean = null;
    public BlastBean getBlastBean() { return this.bean; }
    private GlobusProxy proxy = null;

    public BlastJobListener(BlastBean bean, GlobusProxy proxy) {
        this.bean = bean;
        this.proxy = proxy;
    }

    public void statusChanged(GramJob job) {

        System.err.println(getClassName() + "> GramJobRun::status[" + job.getStatus() + "]");
        System.err.println(getClassName() + "> GramJobRun::status string[" + job.getStatusAsString() + "]");
        System.err.println(getClassName() + "> GramJobRun::rsl[" + job.getRSL() + "]");
        System.err.println(getClassName() + "> GramJobRun::IDAsString[" + job.getIDAsString() + "]");
        if ( job.getStatus() == 4 ) {
            bean.setFinishDate( (new java.util.Date()).toString() );
            System.err.println("Fail Data:[" + bean.getFinishDate() + "]");
            bean.setJobStatus("failed");
        }

        if ( job.getStatus() == 8) {
            try {
                bean.setJobStatus("transfer");
                // setting Local File
                File localFile = new File(
                        CommonProperties.getProperty("LOCAL_WORK_DIR") +
                        System.getProperty("file.separator") +
                        new File(bean.getResultFileName()).getName()
                        );

                // setting FTP Client
                FTPClient ftpClient = FtpClientFactory.createFTPClient(bean.getHostName(), this.proxy);

                // Get Result File
                ftpClient.setType(FTPClient.ASCII);
                ftpClient.get( bean.getResultFileName(), localFile, this);

                // release FTP Client
                FtpClientFactory.releaseFTPClient(ftpClient);

                // transform Clustalw Result File to HTML
                bean.setJobStatus("fix");
                this.remakeResultFile(localFile);

                bean.setFinishDate( (new java.util.Date()).toString() );
                System.err.println("Finish Data:[" + bean.getFinishDate() + "]");

                bean.setJobStatus("finish");
                ActivateJobHolder.remove(this);

            } catch ( Exception e) {
                bean.setFinishDate( (new java.util.Date()).toString() );
                System.err.println("Fail Data:[" + bean.getFinishDate() + "]");
                bean.setJobStatus("failed");
                ActivateJobHolder.remove(this);
                e.printStackTrace();
            }
        }
    }

    private final void remakeResultFile(File file) {
        try {
            StringBuffer fileBody = new StringBuffer();
            fileBody.append("<html><head><title>BLAST Search Results</title>\n");
            fileBody.append("<meta http-equiv=\"Content-Style-Type\" content=\"text/css\">\n");
            fileBody.append("<link rel=\"stylesheet\" href=\"guide.css\">\n");
            fileBody.append("</head>\n<body>\n");

            File tmpFile = new File(file.getPath() + ".bak");
            BufferedWriter bw = new BufferedWriter( new FileWriter(tmpFile) );
            bw.write(fileBody.toString());

            String tmp = "";
            boolean flag = false;
            boolean childFlag = false;

            // Read all  that open result file
            BufferedReader br = new BufferedReader( new FileReader(file) );
            while ( (tmp = br.readLine()) != null ) {
                if ( !flag && (tmp.toLowerCase().indexOf("<pre>") >= 0 || tmp.toLowerCase().indexOf("</pre>") >= 0) ) {
                    flag = true;
                }

                if ( flag && tmp.indexOf("Score    E") >= 0 ) {
                    bw.write("<form action=\"/guide/ClustalwServlet\" method=\"post\" name=\"myform\">\n");
                    bw.write("<input type=\"submit\">\n");
                    bw.write("<input type=\"hidden\" name=\"Function\" value=\"Blast\">\n");
                    bw.write("<input type=\"hidden\" name=\"Org\" value=\"" + bean.getSeqFileName() +"\">\n");
                    childFlag = true;
                }

                if ( childFlag && tmp.toLowerCase().indexOf("</pre>") >= 0 ) {
                    bw.write("</form>\n");
                    childFlag = false;
                }

                if ( childFlag )
                    tmp = remakeJobLine(tmp);

                if ( flag ) {
                    bw.write(tmp + "\n");
                }
            }
            br.close();
            bw.flush();
            bw.close();

            file.delete();
            tmpFile.renameTo(file);

        } catch ( Exception ex ) {
            ex.printStackTrace();
        }
    }

    private final String remakeJobLine(String line) {
        if ( line.indexOf("\"") < 0 )
            return line;

        String value = line.substring(line.indexOf("\"") + 1, line.lastIndexOf("\""));
        StringBuffer buf = new StringBuffer();
        buf.append("<input type=\"checkbox\" name=\"SeqList\" value=\"");
        buf.append(value);
        buf.append("\">");
        buf.append(line);
        return new String(buf);
    }

    public void transfer(int transferedBytes, int totalBytes, String from, String to) {
        //System.err.println(getClassName() + "> bytes:[" + transferedBytes + "/" + totalBytes + "]");
        //System.err.println(getClassName() + "> from:[" + from + "]/to:[" + to + "]");
    }
    public void transferError(String from, String to, Exception exception) {
        System.err.println(getClassName() + "> from:[" + from + "]/to:[" + to + "]");
        exception.printStackTrace();
    }

    /**
     * This Class Name getter
     * @return String This Class Name
     */
    protected final String getClassName() {
        return getClassName(this);
    }

    /**
     * Class Name getter
     * @param obj Object
     * @return String Class Name
     */
    protected final String getClassName(Object obj) {
        String className = obj.getClass().getName();
        return className.substring(className.lastIndexOf('.') + 1);
    }
}
