package jp.co.mki.guide.model;

import java.io.*;
import org.globus.security.GlobusProxy;
import org.globus.gram.GramJob;
import org.globus.gram.GramJobRun;
import org.globus.gram.GramJobListener;
import org.globus.io.ftp.FTPClient;
import org.globus.io.ftp.GridFTPClient;
import org.globus.io.ftp.TransferProgressListener;
import jp.co.mki.guide.bean.*;
import jp.co.mki.guide.common.*;
import jp.co.mki.guide.servlet.*;
import jp.co.mki.guide.util.RSLAnalyzer;

/**
 * Blast Model Class
 *  - GUIDE on WEB SYSTEM -
 * @author Y.KIDO
 * @version 0.0.1
 */
public class BlastModel extends AuthenticationServlet  {

    private String sequentialNo = "";
    private BlastBean blastBean;
    private BlastJobListener listener;

    /**
     * Blast View execute method.
     */
    public void execute() throws Exception {
        BlastBean bean = new BlastBean();
        //super.paramToDataBean(bean);
        super.multiPartToDataBean(bean);

        if ( bean.getSeqLoadType() != null && bean.getSeqLoadType().equals("FILE") ) {
            bean.setSeqText(bean.getSeqFileName());
            bean.setSeqFileName("");
        }

        CommonHosts.setSelectedHost(bean.getHostName());
        bean.setHostName( CommonProperties.getProperty("DEFALUT_GRAM_SERVER") );

        // first visit
        if ( bean.getFunction() == null || bean.getFunction().equals("") || bean.getFunction().equals("LOGIN")) {
            req.setAttribute("blastBean", bean);
            req.getRequestDispatcher("./blast.jsp").forward(super.req, super.res);
            return;
        }

        // Create Job
        this.sequentialNo = super.makeUniqueString();
        bean.setJobID("BLAST" + this.sequentialNo );
        bean.setResultFileName( CommonProperties.getProperty("HOST_WORK_DIR") + "/" + bean.getJobID() + ".html" );
        this.sendSeqFile(bean);

        // Create RSL
        RSLAnalyzer rslObj = new RSLAnalyzer( makeRSLString(bean) );
        System.err.println( "[" + rslObj.getRslString() + "]");

        // Setup Job Object
        System.err.println("hostname:[" + bean.getHostName() + "]");
        GramJobRun gjr = new GramJobRun(rslObj.getRslString(),
                                        bean.getHostName());
        bean.setJobStatus("running");

        // Add GramJobListener
        listener = new BlastJobListener(bean, (GlobusProxy) req.getSession().getAttribute("PROXY") );
        ActivateJobHolder.add(listener);
        JobDatabase.add(bean);

        gjr.addListener( listener );
        bean.setAcceptDate( (new java.util.Date()).toString() );
        System.err.println("Accept Data:[" + bean.getAcceptDate() + "]");
        // Run Job
        gjr.run();

        super.req.getRequestDispatcher("./ResultServlet").forward(super.req, super.res);
    }

    /**
     * Make and send sequence file
     */
    private final void sendSeqFile(BlastBean bean) {
        File file = new File( CommonProperties.getProperty("LOCAL_WORK_DIR") +
                        System.getProperty("file.separator") +
                        "BLAST" + this.sequentialNo + ".seq");
        try {
            BufferedWriter writer = new BufferedWriter(new FileWriter(file));
            writer.write(bean.getSeqText());
            writer.flush();
            writer.close();
        } catch ( Exception ex ) {
            ex.printStackTrace();
        }

        try {
            //make ftp client
            FTPClient ftpClient = FtpClientFactory.createFTPClient(bean.getHostName(), (GlobusProxy) this.req.getAttribute("PROXY"));

            // Send Sequence file
            ftpClient.setType(FTPClient.ASCII);
            ftpClient.changeDir( CommonProperties.getProperty("HOST_WORK_DIR") );
            ftpClient.put(file, file.getName(), false, null);

            FtpClientFactory.releaseFTPClient(ftpClient);
        } catch ( Exception ex ) {
            ex.printStackTrace();
        }

        bean.setSeqFileName( file.getPath() );
    }

    /**
     * Make RSL String
     * @return String RSL String
     */
    private final String makeRSLString(BlastBean bean) throws Exception {

        String createdRSL = null;

        String blastBin = CommonProperties.getProperty("BLAST_BIN");
        String hostPath = CommonProperties.getProperty("HOST_WORK_DIR");
        String args = CommonProperties.getProperty("BLAST_ARGUMENTS");

        int proc_cnt = 1;

        // add Program Name
        args += " -p " + bean.getProgramName();

        // add DB File Name
        args += " -d " + bean.getDBName();

        // add Expect Value
        if ( bean.getExpect() != null && !bean.getExpect().equals("") )
            args += " -e " + bean.getExpect();


        // add Matrix File Name
        if ( bean.getProgramName().equals("blastn") || bean.getProgramName().equals("tblastn") ) {
            // none
        // PROTEIN or PROTEIN vs NUCLEIC
        } else if ( !bean.getMatrixFileName().equals("default") ) {
            args += " -M " + bean.getMatrixFileName();
        }

        // add Sequence File Name
        if ( bean.getSeqFileName() != null && !bean.getSeqFileName().equals("") )
            args += " -i " + hostPath + bean.getSeqFileName().substring( bean.getSeqFileName().lastIndexOf("/") );

        // add Result File Name
        if ( bean.getResultFileName() != null && !bean.getResultFileName().equals("") )
            args += " -o " + bean.getResultFileName();

        // add OpenGap Value
        if( bean.getOpenGap() != null && !bean.getOpenGap().equals("default") )
            args += " -G " + bean.getOpenGap();

        // add ExtendGap Value
        if( bean.getExtendGap() != null && !bean.getExtendGap().equals("default") )
            args += " -E " + bean.getExtendGap();

        // add Strand Value
        if ( bean.getStrand() != null && !bean.getStrand().equals("default") )
            args += " -S " + bean.getStrand();

        // add trance HTML option
        args += " -T T";

        if ( bean.getMoreOption() != null && !bean.getMoreOption().equals("") )
            args += " " + bean.getMoreOption();

        createdRSL = "&(executable = " + blastBin + ")\n";
        createdRSL += "(arguments = " + args + ")\n";

        boolean endFlag = false;
        boolean firstFlag = true;
        for ( int i = 0; ; i++) {
            String tmp = CommonProperties.getProperty("BLAST_ENV_" + i);
            if ( tmp == null || tmp.equals("") || tmp.equals("EOA") ) {
                if ( endFlag ) createdRSL += ")\n";
                break;
            } else {
                if ( firstFlag ) createdRSL += "(environment = ";
                endFlag = true;
                firstFlag = false;
                createdRSL += "(" + tmp + ")";
            }
        }

        return createdRSL;
    }

}