package jp.co.mki.guide.model;

import java.io.*;
import org.globus.security.*;
import org.globus.gram.GramJobListener;
import org.globus.gram.GramJob;
import org.globus.io.ftp.FTPClient;
import org.globus.io.ftp.GridFTPClient;
import org.globus.io.ftp.TransferProgressListener;
import jp.co.mki.guide.common.*;
import jp.co.mki.guide.bean.ClustalwBean;

/**
 * @author Y.KIDO
 * @version 0.0.1
 */
public class ClustalwJobListener implements GramJobListener, TransferProgressListener {

    private ClustalwBean bean = null;
    public ClustalwBean getClustlwBean() { return this.bean; }
    private GlobusProxy proxy = null;

    public ClustalwJobListener(ClustalwBean bean, GlobusProxy proxy) {
        this.bean = bean;
        this.proxy = proxy;
    }

    public void statusChanged(GramJob job) {

        System.err.println(getClassName() + "> GramJobRun::status[" + job.getStatus() + "]");
        System.err.println(getClassName() + "> GramJobRun::status string[" + job.getStatusAsString() + "]");
        System.err.println(getClassName() + "> GramJobRun::rsl[" + job.getRSL() + "]");
        System.err.println(getClassName() + "> GramJobRun::IDAsString[" + job.getIDAsString() + "]");
        if ( job.getStatus() == 4 ) {
            bean.setFinishDate( (new java.util.Date()).toString() );
            System.err.println("Fail Data:[" + bean.getFinishDate() + "]");
            bean.setJobStatus("failed");
        }

        if ( job.getStatus() == 8) {
            try {
                String resultFile = bean.getResultFileName();

                bean.setJobStatus("transfer");
                // setting Local File
                File localFile = new File( bean.getResultFileNameForLocal() );

                // setting FTP Client
                FTPClient ftpClient = FtpClientFactory.createFTPClient(bean.getHostName(), this.proxy);

                // Get Result File
                ftpClient.setType(FTPClient.ASCII);
                ftpClient.get( resultFile.substring(0, resultFile.lastIndexOf(".")) + ".aln", localFile, this);

                // release FTP Client
                FtpClientFactory.releaseFTPClient(ftpClient);

                // transform Clustalw Result File to HTML
                bean.setJobStatus("fix");
                this.remakeResultFile(localFile);

                bean.setFinishDate( (new java.util.Date()).toString() );
                System.err.println("Finish Data:[" + bean.getFinishDate() + "]");

                bean.setJobStatus("finish");
                ActivateJobHolder.remove(this);

            } catch ( Exception e) {
                bean.setJobStatus("error");
                e.printStackTrace();
            }
        }
    }

    private final void remakeResultFile(File file) {
        try {
            StringBuffer fileBody = new StringBuffer();
            String tmp = "";
            fileBody.append("<html><head>\n");
            fileBody.append("<meta http-equiv=\"Content-Style-Type\" content=\"text/css\">\n");
            fileBody.append("<link rel=\"stylesheet\" href=\"guide.css\">\n");
            fileBody.append("</head><body><pre>\n");

            // Read result that open result file
            BufferedReader br = new BufferedReader( new FileReader(file) );
            while ( (tmp = br.readLine()) != null )
                fileBody.append(tmp).append("\n");
            br.close();

            fileBody.append("</pre></body></html>\n");

            BufferedWriter bw = new BufferedWriter( new FileWriter( file ) );
            bw.write(fileBody.toString());
            bw.flush();
            bw.close();
        } catch ( Exception ex ) {
            ex.printStackTrace();
        }
    }

    public void transfer(int transferedBytes, int totalBytes, String from, String to) {
        //System.err.println(getClassName() + "> bytes:[" + transferedBytes + "/" + totalBytes + "]");
        //System.err.println(getClassName() + "> from:[" + from + "]/to:[" + to + "]");
    }
    public void transferError(String from, String to, Exception exception) {
        System.err.println(getClassName() + "> from:[" + from + "]/to:[" + to + "]");
        exception.printStackTrace();
    }

    /**
     * This Class Name getter
     * @return String This Class Name
     */
    protected final String getClassName() {
        return getClassName(this);
    }

    /**
     * Class Name getter
     * @param obj Object
     * @return String Class Name
     */
    protected final String getClassName(Object obj) {
        String className = obj.getClass().getName();
        return className.substring(className.lastIndexOf('.') + 1);
    }
}