package jp.co.mki.guide.model;

import java.io.*;
import java.net.*;
import java.util.*;
import org.globus.security.GlobusProxy;
import org.globus.gram.GramJob;
import org.globus.gram.GramJobRun;
import org.globus.gram.GramJobListener;
import org.globus.io.ftp.FTPClient;
import org.globus.io.ftp.GridFTPClient;
import org.globus.io.ftp.TransferProgressListener;
import jp.co.mki.guide.bean.*;
import jp.co.mki.guide.servlet.*;
import jp.co.mki.guide.common.*;
import jp.co.mki.guide.util.*;

/**
 * ClustalW Model Class
 *  - GUIDE on WEB SYSTEM -
 * @author Y.KIDO
 * @version 0.0.1
 */
public class ClustalwModel extends AuthenticationServlet {

    private String sequentialNo = "";

    public ClustalwModel() {}

    public void execute() throws Exception {
        ClustalwBean bean = new ClustalwBean();
        // update kido. 2003.07.22
        if ( super.req.getParameterMap().size() > 0 )
            super.paramToDataBean(bean);
        else
            super.multiPartToDataBean(bean);
        if ( bean.getSeqLoadType() != null && bean.getSeqLoadType().equals("FILE") ) {
            bean.setSeqText(bean.getSeqFileName());
            bean.setSeqFileName("");
        }

        CommonHosts.setSelectedHost(bean.getHostName());
        bean.setHostName( CommonProperties.getProperty("DEFALUT_GRAM_SERVER") );

        // first visit
        if ( bean.getFunction() == null || bean.getFunction().equals("") ) {
            req.setAttribute("clustalwBean", bean);
            req.getRequestDispatcher("./clustalw.jsp").forward(super.req, super.res);
            return;
        }

        // from blast result page
        if ( bean.getFunction() != null && bean.getFunction().equals("Blast") ) {
            bean.setSeqText( getFastaFormat() );

            req.setAttribute("clustalwBean", bean);
            req.getRequestDispatcher("./clustalw.jsp").forward(super.req, super.res);
            return;
        }

        // Create Job
        this.sequentialNo = super.makeUniqueString();
        bean.setJobID("CLW" + this.sequentialNo );
        bean.setResultFileName( CommonProperties.getProperty("HOST_WORK_DIR") + "/" + bean.getJobID() + ".html" );
        this.sendSeqFile(bean);

        // Create RSL
        RSLAnalyzer rslObj = new RSLAnalyzer( makeRSLString(bean) );
        System.err.println( "[" + rslObj.getRslString() + "]");

        // Setup Job Object
        GramJobRun gjr = new GramJobRun(rslObj.getRslString(),
                                        bean.getHostName());
        bean.setJobStatus("running");

        // Add GramJobListener
        ClustalwJobListener listener = new ClustalwJobListener(bean, (GlobusProxy) req.getSession().getAttribute("PROXY") );
        ActivateJobHolder.add(listener);
        JobDatabase.add(bean);

        gjr.addListener( listener );
        bean.setAcceptDate( (new java.util.Date()).toString() );
        System.err.println("Accept Data:[" + bean.getAcceptDate() + "]");

        // Run Job
        gjr.run();

        super.req.getRequestDispatcher("./ResultServlet").forward(super.req, super.res);
    }

    /**
     * Make and send sequence file
     */
    private final void sendSeqFile(ClustalwBean bean) {
        File file = new File( CommonProperties.getProperty("LOCAL_WORK_DIR") +
                        System.getProperty("file.separator") +
                        "CLW" + this.sequentialNo + ".seq");
        try {
            BufferedWriter writer = new BufferedWriter(new FileWriter(file));
            writer.write(bean.getSeqText());
            writer.flush();
            writer.close();
        } catch ( Exception ex ) {
            ex.printStackTrace();
        }

        try {
            //make ftp client
            FTPClient ftpClient = FtpClientFactory.createFTPClient(bean.getHostName(), (GlobusProxy) this.req.getAttribute("PROXY"));

            // Send Sequence file
            ftpClient.setType(FTPClient.ASCII);
            ftpClient.changeDir( CommonProperties.getProperty("HOST_WORK_DIR") );
            ftpClient.put(file, file.getName(), false, null);

            FtpClientFactory.releaseFTPClient(ftpClient);
        } catch ( Exception ex ) {
            ex.printStackTrace();
        }

        bean.setSeqFileName( file.getPath() );
    }

    /**
     * Make RSL String
     * @return String RSL String
     */
    private final String makeRSLString(ClustalwBean bean) throws Exception {

        String createdRSL = null;
        String clustalwBin = CommonProperties.getProperty("CLUSTALW_BIN");
        String hostPath = CommonProperties.getProperty("HOST_WORK_DIR");

        StringBuffer args = new StringBuffer( CommonProperties.getProperty("CLUSTALW_ARGUMENTS") );

        // add sequence file name
        String seqFile = bean.getSeqFileName();
        args.append( CommonProperties.getProperty("HOST_WORK_DIR") +
                    "/" +
                    seqFile.substring( seqFile.lastIndexOf("/") + 1 ) );

        if ( bean.getAlignment() != null && bean.getAlignment().equals("full") )
            args.append(" -align");
        else
            args.append(" -quicktree");

        //Fast Setting
        if ( bean.getWordSize() != null && !bean.getWordSize().equals("default") )
            args.append(" -ktuple=").append( bean.getWordSize() );

        if ( bean.getScoreType() != null && !bean.getScoreType().equals("default") )
            args.append(" -score=").append( bean.getScoreType() );

        if ( bean.getWindowLength() != null && !bean.getWindowLength().equals("default") )
            args.append(" -window=").append( bean.getWindowLength() );

        if ( bean.getTopDiag() != null && !bean.getTopDiag().equals("default") )
            args.append(" -topdiags=").append( bean.getTopDiag() );

        if ( bean.getPairGap() != null && !bean.getPairGap().equals("default") )
            args.append(" -pairgap=").append( bean.getPairGap() );

        // Gap Setting
        if ( bean.getGapOpen() != null && !bean.getGapOpen().equals("default") )
            args.append(" -gapopen=").append( bean.getGapOpen() );

        if ( bean.getEndGaps() != null && !bean.getEndGaps().equals("default") )
            args.append(" -endgaps=").append( bean.getEndGaps() );

        if ( bean.getGapExtension() != null && !bean.getGapExtension().equals("default") )
            args.append(" -gapext=").append( bean.getGapExtension() );

        if ( bean.getGapDistances() != null && !bean.getGapDistances().equals("default") )
            args.append(" -gapdist=").append( bean.getGapDistances() );

        // Commons
        args.append(" -outorder=").append( bean.getOutputOrder().toLowerCase() );

        if ( bean.getMatrix() != null && !bean.getMatrix().equals("default") )
            args.append(" -pwmatix=").append( bean.getMatrix().toLowerCase() );

        if ( bean.getOutputFormat() != null && !bean.getOutputFormat().equals("default") )
            args.append(" -output=").append( bean.getOutputFormat().toLowerCase() );

        createdRSL = "&(executable = " + clustalwBin + ")\n";
        createdRSL += "(arguments = " + args + ")\n";

        boolean endFlag = false;
        boolean firstFlag = true;
        for ( int i = 0; ; i++) {
            String tmp = CommonProperties.getProperty("CLUSTALW_ENV_" + i);
            if ( tmp == null || tmp.equals("") || tmp.equals("EOA") ) {
                if ( endFlag ) createdRSL += ")\n";
                break;
            } else {
                if ( firstFlag ) createdRSL += "(environment = ";
                endFlag = true;
                firstFlag = false;
                createdRSL += "(" + tmp + ")";
            }
        }
        System.out.println( createdRSL );
        return createdRSL;
    }

    private final String getFastaFormat() {
        String[] urls = req.getParameterValues("SeqList");
        StringBuffer buf = new StringBuffer();
        for ( int i = 0; i < urls.length; i++ )
            buf.append( getFastaFormat(urls[i]) );
        return new String(buf);
    }

    private final String getFastaFormat(String url) {

        BufferedReader htmlReader = null;
        StringBuffer buf = new StringBuffer();

        try{
            URL soleURL = new URL(url);
            htmlReader = new BufferedReader(new InputStreamReader(soleURL.openStream()));
            String tmp = null;

            while ( (tmp = htmlReader.readLine()) != null && !tmp.trim().startsWith("ACCESSION") );
            StringTokenizer st = new StringTokenizer(tmp.trim());
            st.nextToken();
            // Get Accession No
            buf.append("> ").append( st.nextToken() ).append("\n");

            while ( (tmp = htmlReader.readLine()) != null && !tmp.trim().startsWith("ORIGIN") );

            while ( (tmp = htmlReader.readLine()) != null && !tmp.startsWith("//") ) {

                st = new StringTokenizer(tmp.trim()," ");
                st.nextElement();

                // Get Genome Sequence
                while ( st.hasMoreTokens() )
                    buf.append( st.nextToken() );
            }
            buf.append("\n");

        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try {
                if ( htmlReader != null ) htmlReader.close();
            } catch ( Exception ex ) { ex.printStackTrace(); }
        }
        return new String(buf);
    }
}